@@ -1,11 +1,9 @@
 /*
  * extractExternal.cpp
- * $Revision: 43084 $
- * $Date: 2014-04-15 09:15:14 -0500 (Tue, 15 Apr 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 2006-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2006-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -160,8 +158,8 @@ private:
 	    data = new char[length = _length];
 	}
 	*(unsigned*)data = length;
-	memcpy(data + sizeof(unsigned), _data + sizeof(unsigned),
-	       length - sizeof(unsigned));
+	KMP_MEMCPY(data + sizeof(unsigned), _data + sizeof(unsigned),
+	           length - sizeof(unsigned));
 	makeDirectory();
     }
 public:
@@ -213,7 +211,7 @@ public:
 	    size_t l = str.size();
 	    if(l > 8) {
 		directory.insert(make_pair(str, p - data));
-		memcpy(p, str.c_str(), l);
+		KMP_MEMCPY(p, str.c_str(), l);
 		p[l] = 0;
 		p += l + 1;
 	    }
@@ -232,7 +230,7 @@ public:
 	if(str.size() <= 8) {
 	    // encoded directly
 	    ((char*)&r)[7] = 0;
-	    strncpy((char*)&r, str.c_str(), 8);
+	    KMP_STRNCPY_S((char*)&r, sizeof(r), str.c_str(), 8);
 	    return r;
 	} else {
 	    // represented as index into table
