@@ -1,11 +1,9 @@
 /*
- * kmp_io.c -- RTL IO
- * $Revision: 43236 $
- * $Date: 2014-06-04 16:42:35 -0500 (Wed, 04 Jun 2014) $
+ * KMP_IO.c -- RTL IO
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -190,9 +188,9 @@ __kmp_vprintf( enum kmp_io __kmp_io, char const * format, va_list ap )
         int chars = 0;
 
         #ifdef KMP_DEBUG_PIDS
-            chars = sprintf( db, "pid=%d: ", (kmp_int32)getpid() );
+            chars = KMP_SNPRINTF( db, __kmp_debug_buf_chars, "pid=%d: ", (kmp_int32)getpid() );
         #endif
-        chars += vsprintf( db, format, ap );
+        chars += KMP_VSNPRINTF( db, __kmp_debug_buf_chars, format, ap );
 
         if ( chars + 1 > __kmp_debug_buf_chars ) {
             if ( chars + 1 > __kmp_debug_buf_warn_chars ) {
