@@ -1,11 +1,9 @@
 /*
  * kmp_str.c -- String manipulation routines.
- * $Revision: 43084 $
- * $Date: 2014-04-15 09:15:14 -0500 (Tue, 15 Apr 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -133,7 +131,7 @@ __kmp_str_buf_reserve(
             if ( buffer->str == NULL ) {
 		KMP_FATAL( MemoryAllocFailed );
             }; // if
-            memcpy( buffer->str, buffer->bulk, buffer->used + 1 );
+            KMP_MEMCPY_S( buffer->str, buffer->size, buffer->bulk, buffer->used + 1 );
         } else {
             buffer->str = (char *) KMP_INTERNAL_REALLOC( buffer->str, buffer->size );
             if ( buffer->str == NULL ) {
@@ -163,7 +161,7 @@ __kmp_str_buf_detach(
         if ( buffer->str == NULL ) {
 		KMP_FATAL( MemoryAllocFailed );
         }; // if
-        memcpy( buffer->str, buffer->bulk, buffer->used + 1 );
+        KMP_MEMCPY_S( buffer->str, buffer->size, buffer->bulk, buffer->used + 1 );
     }; // if
 
 } // __kmp_str_buf_detach
@@ -194,7 +192,7 @@ __kmp_str_buf_cat(
     KMP_DEBUG_ASSERT( str != NULL );
     KMP_DEBUG_ASSERT( len >= 0 );
     __kmp_str_buf_reserve( buffer, buffer->used + len + 1 );
-    memcpy( buffer->str + buffer->used, str, len );
+    KMP_MEMCPY( buffer->str + buffer->used, str, len );
     buffer->str[ buffer->used + len ] = 0;
     buffer->used += len;
     KMP_STR_BUF_INVARIANT( buffer );
@@ -232,7 +230,7 @@ __kmp_str_buf_vprint(
                 __va_copy( _args, args );  // Make copy of args.
                 #define args _args         // Substitute args with its copy, _args.
             #endif // KMP_OS_WINDOWS
-            rc = vsnprintf( buffer->str + buffer->used, free, format, args );
+            rc = KMP_VSNPRINTF( buffer->str + buffer->used, free, format, args );
             #if ! KMP_OS_WINDOWS
                 #undef args                // Remove substitution.
                 va_end( _args );
@@ -523,7 +521,7 @@ __kmp_str_format(           // Allocated string.
 
         // Try to format string.
         va_start( args, format );
-        rc = vsnprintf( buffer, size, format, args );
+        rc = KMP_VSNPRINTF( buffer, size, format, args );
         va_end( args );
 
         // No errors, string has been formatted.
