@@ -1,11 +1,9 @@
 /*
  * kmp_taskq.c -- TASKQ support for OpenMP.
- * $Revision: 43389 $
- * $Date: 2014-08-11 10:54:01 -0500 (Mon, 11 Aug 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -69,7 +67,11 @@ __kmp_taskq_eo( int *gtid_ref, int *cid_ref, ident_t *loc_ref )
     kmp_taskq_t       *tq   = & __kmp_threads[gtid] -> th.th_team -> t.t_taskq;
 
     if ( __kmp_env_consistency_check )
+#if KMP_USE_DYNAMIC_LOCK
+        __kmp_push_sync( gtid, ct_ordered_in_taskq, loc_ref, NULL, 0 );
+#else
         __kmp_push_sync( gtid, ct_ordered_in_taskq, loc_ref, NULL );
+#endif
 
     if ( ! __kmp_threads[ gtid ]-> th.th_team -> t.t_serialized ) {
         KMP_MB();       /* Flush all pending memory write invalidates.  */
