@@ -1,11 +1,9 @@
 /*
  * kmp_threadprivate.c -- OpenMP threadprivate support library
- * $Revision: 42951 $
- * $Date: 2014-01-21 14:41:41 -0600 (Tue, 21 Jan 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -135,7 +133,7 @@ __kmp_init_common_data( void *pc_addr, size_t pc_size )
     for (i = pc_size;  i > 0; --i) {
         if (*p++ != '\0') {
             d->data = __kmp_allocate( pc_size );
-            memcpy( d->data, pc_addr, pc_size );
+            KMP_MEMCPY( d->data, pc_addr, pc_size );
             break;
         }
     }
@@ -158,7 +156,7 @@ __kmp_copy_common_data( void *pc_addr, struct private_data *d )
             if (d->data == 0)
                 memset( & addr[ offset ], '\0', d->size );
             else
-                memcpy( & addr[ offset ], d->data, d->size );
+                KMP_MEMCPY( & addr[ offset ], d->data, d->size );
             offset += d->size;
         }
     }
