@@ -1,11 +1,9 @@
 /*
  * kmp_utility.c -- Utility routines for the OpenMP support library.
- * $Revision: 42951 $
- * $Date: 2014-01-21 14:41:41 -0600 (Tue, 21 Jan 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -253,16 +251,9 @@ __kmp_query_cpuid( kmp_cpuinfo_t *p )
         }
 #endif /* KMP_DEBUG */
 
-        __kmp_ht_capable = FALSE;
         if ( (buf.edx >> 28) & 1 ) {
-
-            /* HT - Processor is HT Enabled (formerly JT) */
-            __kmp_ht_capable = TRUE;
-
             /* Bits 23-16: Logical Processors per Physical Processor (1 for P4) */
             log_per_phy = data[ 2 ];
-            __kmp_ht_log_per_phy = log_per_phy;
-
             p->apic_id     = data[ 3 ]; /* Bits 31-24: Processor Initial APIC ID (X) */
             KA_TRACE( trace_level, (" HT(%d TPUs)", log_per_phy ) );
 
@@ -342,12 +333,12 @@ __kmp_expand_host_name( char *buffer, size_t size )
 	DWORD	s = size;
 
 	if (! GetComputerNameA( buffer, & s ))
-	    strcpy( buffer, unknown );
+	    KMP_STRCPY_S( buffer, size, unknown );
     }
 #else
     buffer[size - 2] = 0;
     if (gethostname( buffer, size ) || buffer[size - 2] != 0)
-	strcpy( buffer, unknown );
+	KMP_STRCPY_S( buffer, size, unknown );
 #endif
 }
 
@@ -402,7 +393,7 @@ __kmp_expand_file_name( char *result, size_t rlen, char *pattern )
 		case 'h':
 		    {
 			__kmp_expand_host_name( buffer, sizeof( buffer ) );
-			strncpy( pos,  buffer, end - pos + 1);
+			KMP_STRNCPY( pos,  buffer, end - pos + 1);
 			if(*end == 0) {
 			    while ( *pos )
 				++pos;
@@ -414,7 +405,7 @@ __kmp_expand_file_name( char *result, size_t rlen, char *pattern )
 		case 'P':
 		case 'p':
 		    {
-			snp_result = snprintf( pos, end - pos + 1, "%0*d", cpu_width, __kmp_dflt_team_nth );
+			snp_result = KMP_SNPRINTF( pos, end - pos + 1, "%0*d", cpu_width, __kmp_dflt_team_nth );
 			if(snp_result >= 0 && snp_result <= end - pos) {
 			    while ( *pos )
 				++pos;
@@ -427,7 +418,7 @@ __kmp_expand_file_name( char *result, size_t rlen, char *pattern )
 		case 'i':
 		    {
 			pid_t id = getpid();
-			snp_result = snprintf( pos, end - pos + 1, "%0*d", width, id );
+			snp_result = KMP_SNPRINTF( pos, end - pos + 1, "%0*d", width, id );
 			if(snp_result >= 0 && snp_result <= end - pos) {
 			    while ( *pos )
 				++pos;
