@@ -1,11 +1,9 @@
 /*
  * z_Linux_util.c -- platform specific routines.
- * $Revision: 43473 $
- * $Date: 2014-09-26 15:02:57 -0500 (Fri, 26 Sep 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -55,7 +53,7 @@
 
 #if KMP_OS_LINUX && !KMP_OS_CNK
 # include <sys/sysinfo.h>
-# if KMP_OS_LINUX && (KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_ARM)
+# if KMP_OS_LINUX && (KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_ARM || KMP_ARCH_AARCH64)
 // We should really include <futex.h>, but that causes compatibility problems on different
 // Linux* OS distributions that either require that you include (or break when you try to include)
 // <pci/types.h>.
@@ -82,7 +80,7 @@
 #include <fcntl.h>
 
 // For non-x86 architecture
-#if KMP_COMPILER_GCC && !(KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_PPC64)
+#if KMP_COMPILER_GCC && !(KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_PPC64 || KMP_ARCH_AARCH64)
 # include <stdbool.h>
 # include <ffi.h>
 #endif
@@ -122,7 +120,7 @@ static kmp_mutex_align_t   __kmp_wait_mx;
 static void
 __kmp_print_cond( char *buffer, kmp_cond_align_t *cond )
 {
-    sprintf( buffer, "(cond (lock (%ld, %d)), (descr (%p)))",
+    KMP_SNPRINTF( buffer, 128, "(cond (lock (%ld, %d)), (descr (%p)))",
                       cond->c_cond.__c_lock.__status, cond->c_cond.__c_lock.__spinlock,
                       cond->c_cond.__c_waiting );
 }
@@ -156,6 +154,18 @@ __kmp_print_cond( char *buffer, kmp_cond_align_t *cond )
 #    error Wrong code for getaffinity system call.
 #   endif /* __NR_sched_getaffinity */
 
+#  elif KMP_ARCH_AARCH64
+#   ifndef __NR_sched_setaffinity
+#    define __NR_sched_setaffinity  122
+#   elif __NR_sched_setaffinity != 122
+#    error Wrong code for setaffinity system call.
+#   endif /* __NR_sched_setaffinity */
+#   ifndef __NR_sched_getaffinity
+#    define __NR_sched_getaffinity  123
+#   elif __NR_sched_getaffinity != 123
+#    error Wrong code for getaffinity system call.
+#   endif /* __NR_sched_getaffinity */
+
 #  elif KMP_ARCH_X86_64
 #   ifndef __NR_sched_setaffinity
 #    define __NR_sched_setaffinity  203
@@ -236,7 +246,7 @@ __kmp_affinity_bind_thread( int which )
     KMP_ASSERT2(KMP_AFFINITY_CAPABLE(),
       "Illegal set affinity operation when not capable");
 
-    kmp_affin_mask_t *mask = (kmp_affin_mask_t *)alloca(__kmp_affin_mask_size);
+    kmp_affin_mask_t *mask = (kmp_affin_mask_t *)KMP_ALLOCA(__kmp_affin_mask_size);
     KMP_CPU_ZERO(mask);
     KMP_CPU_SET(which, mask);
     __kmp_set_system_affinity(mask, TRUE);
@@ -286,7 +296,7 @@ __kmp_affinity_determine_capable(const char *env_var)
                 __kmp_msg_null
             );
         }
-        __kmp_affin_mask_size = 0;  // should already be 0
+        KMP_AFFINITY_DISABLE();
         KMP_INTERNAL_FREE(buf);
         return;
     }
@@ -314,11 +324,11 @@ __kmp_affinity_determine_capable(const char *env_var)
                         __kmp_msg_null
                     );
                 }
-                __kmp_affin_mask_size = 0;  // should already be 0
+                KMP_AFFINITY_DISABLE();
                 KMP_INTERNAL_FREE(buf);
             }
             if (errno == EFAULT) {
-                __kmp_affin_mask_size = gCode;
+                KMP_AFFINITY_ENABLE(gCode);
                 KA_TRACE(10, ( "__kmp_affinity_determine_capable: "
                   "affinity supported (mask size %d)\n",
                   (int)__kmp_affin_mask_size));
@@ -362,7 +372,7 @@ __kmp_affinity_determine_capable(const char *env_var)
                         __kmp_msg_null
                     );
                 }
-                __kmp_affin_mask_size = 0;  // should already be 0
+                KMP_AFFINITY_DISABLE();
                 KMP_INTERNAL_FREE(buf);
                 return;
             }
@@ -393,12 +403,12 @@ __kmp_affinity_determine_capable(const char *env_var)
                         __kmp_msg_null
                     );
                 }
-                __kmp_affin_mask_size = 0;  // should already be 0
+                KMP_AFFINITY_DISABLE();
                 KMP_INTERNAL_FREE(buf);
                 return;
             }
             if (errno == EFAULT) {
-                __kmp_affin_mask_size = gCode;
+                KMP_AFFINITY_ENABLE(gCode);
                 KA_TRACE(10, ( "__kmp_affinity_determine_capable: "
                   "affinity supported (mask size %d)\n",
                    (int)__kmp_affin_mask_size));
@@ -414,7 +424,7 @@ __kmp_affinity_determine_capable(const char *env_var)
     //
     // Affinity is not supported
     //
-    __kmp_affin_mask_size = 0;
+    KMP_AFFINITY_DISABLE();
     KA_TRACE(10, ( "__kmp_affinity_determine_capable: "
       "cannot determine mask size - affinity not supported\n"));
     if (__kmp_affinity_verbose || (__kmp_affinity_warnings
@@ -425,61 +435,12 @@ __kmp_affinity_determine_capable(const char *env_var)
     }
 }
 
-
-/*
- * Change thread to the affinity mask pointed to by affin_mask argument
- * and return a pointer to the old value in the old_mask argument, if argument
- * is non-NULL.
- */
-
-void
-__kmp_change_thread_affinity_mask( int gtid, kmp_affin_mask_t *new_mask,
-                                   kmp_affin_mask_t *old_mask )
-{
-    KMP_DEBUG_ASSERT( gtid == __kmp_get_gtid() );
-    if ( KMP_AFFINITY_CAPABLE() ) {
-        int status;
-        kmp_info_t  *th = __kmp_threads[ gtid ];
-
-        KMP_DEBUG_ASSERT( new_mask != NULL );
-
-        if ( old_mask != NULL ) {
-            status = __kmp_get_system_affinity( old_mask, TRUE );
-            int error = errno;
-            if ( status != 0 ) {
-                __kmp_msg(
-                    kmp_ms_fatal,
-                    KMP_MSG( ChangeThreadAffMaskError ),
-                    KMP_ERR( error ),
-                    __kmp_msg_null
-                );
-            }
-        }
-
-        __kmp_set_system_affinity( new_mask, TRUE );
-
-        if (__kmp_affinity_verbose) {
-            char old_buf[KMP_AFFIN_MASK_PRINT_LEN];
-            char new_buf[KMP_AFFIN_MASK_PRINT_LEN];
-            __kmp_affinity_print_mask(old_buf, KMP_AFFIN_MASK_PRINT_LEN, old_mask);
-            __kmp_affinity_print_mask(new_buf, KMP_AFFIN_MASK_PRINT_LEN, new_mask);
-            KMP_INFORM( ChangeAffMask, "KMP_AFFINITY (Bind)", gtid, old_buf, new_buf );
-
-        }
-
-        /* Make sure old value is correct in thread data structures */
-        KMP_DEBUG_ASSERT( old_mask != NULL && (memcmp(old_mask,
-          th->th.th_affin_mask, __kmp_affin_mask_size) == 0) );
-        KMP_CPU_COPY( th->th.th_affin_mask, new_mask );
-    }
-}
-
 #endif // KMP_OS_LINUX && KMP_AFFINITY_SUPPORTED
 
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
-#if KMP_OS_LINUX && (KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_ARM) && !KMP_OS_CNK
+#if KMP_OS_LINUX && (KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_ARM || KMP_ARCH_AARCH64) && !KMP_OS_CNK
 
 int
 __kmp_futex_determine_capable()
@@ -507,6 +468,40 @@ __kmp_futex_determine_capable()
  * use compare_and_store for these routines
  */
 
+kmp_int8
+__kmp_test_then_or8( volatile kmp_int8 *p, kmp_int8 d )
+{
+    kmp_int8 old_value, new_value;
+
+    old_value = TCR_1( *p );
+    new_value = old_value | d;
+
+    while ( ! KMP_COMPARE_AND_STORE_REL8 ( p, old_value, new_value ) )
+    {
+        KMP_CPU_PAUSE();
+        old_value = TCR_1( *p );
+        new_value = old_value | d;
+    }
+    return old_value;
+}
+
+kmp_int8
+__kmp_test_then_and8( volatile kmp_int8 *p, kmp_int8 d )
+{
+    kmp_int8 old_value, new_value;
+
+    old_value = TCR_1( *p );
+    new_value = old_value & d;
+
+    while ( ! KMP_COMPARE_AND_STORE_REL8 ( p, old_value, new_value ) )
+    {
+        KMP_CPU_PAUSE();
+        old_value = TCR_1( *p );
+        new_value = old_value & d;
+    }
+    return old_value;
+}
+
 kmp_int32
 __kmp_test_then_or32( volatile kmp_int32 *p, kmp_int32 d )
 {
@@ -541,7 +536,24 @@ __kmp_test_then_and32( volatile kmp_int32 *p, kmp_int32 d )
     return old_value;
 }
 
-# if KMP_ARCH_X86 || KMP_ARCH_PPC64
+# if KMP_ARCH_X86 || KMP_ARCH_PPC64 || KMP_ARCH_AARCH64
+kmp_int8
+__kmp_test_then_add8( volatile kmp_int8 *p, kmp_int8 d )
+{
+    kmp_int8 old_value, new_value;
+
+    old_value = TCR_1( *p );
+    new_value = old_value + d;
+
+    while ( ! KMP_COMPARE_AND_STORE_REL8 ( p, old_value, new_value ) )
+    {
+        KMP_CPU_PAUSE();
+        old_value = TCR_1( *p );
+        new_value = old_value + d;
+    }
+    return old_value;
+}
+
 kmp_int64
 __kmp_test_then_add64( volatile kmp_int64 *p, kmp_int64 d )
 {
@@ -690,7 +702,7 @@ __kmp_launch_worker( void *thr )
     sigset_t    new_set, old_set;
 #endif /* KMP_BLOCK_SIGNALS */
     void *exit_val;
-    void *padding = 0;
+    void * volatile padding = 0;
     int gtid;
     int error;
 
@@ -739,7 +751,7 @@ __kmp_launch_worker( void *thr )
 
 #if KMP_OS_LINUX || KMP_OS_FREEBSD
     if ( __kmp_stkoffset > 0 && gtid > 0 ) {
-        padding = alloca( gtid * __kmp_stkoffset );
+        padding = KMP_ALLOCA( gtid * __kmp_stkoffset );
     }
 #endif
 
@@ -850,7 +862,7 @@ __kmp_launch_monitor( void *thr )
         interval.tv_nsec = 0;
     } else {
         interval.tv_sec  = 0;
-        interval.tv_nsec = (NSEC_PER_SEC / __kmp_monitor_wakeups);
+        interval.tv_nsec = (KMP_NSEC_PER_SEC / __kmp_monitor_wakeups);
     }
 
     KA_TRACE( 10, ("__kmp_launch_monitor: #2 monitor\n" ) );
@@ -877,9 +889,9 @@ __kmp_launch_monitor( void *thr )
         now.tv_sec  += interval.tv_sec;
         now.tv_nsec += interval.tv_nsec;
 
-        if (now.tv_nsec >= NSEC_PER_SEC) {
+        if (now.tv_nsec >= KMP_NSEC_PER_SEC) {
             now.tv_sec  += 1;
-            now.tv_nsec -= NSEC_PER_SEC;
+            now.tv_nsec -= KMP_NSEC_PER_SEC;
         }
 
         status = pthread_mutex_lock( & __kmp_wait_mx.m_mutex );
@@ -1020,7 +1032,11 @@ __kmp_create_worker( int gtid, kmp_info_t *th, size_t stack_size )
             }; // if
 
             /* Set stack size for this thread now. */
-            stack_size += gtid * __kmp_stkoffset;
+            // Stack compensation for the stack offset.
+            // When the thread is created and __kmp_stkoffset != 0, then
+            // it will alloca(), and the stacks will then
+            // be offset from a page by different values for different threads.
+            stack_size += gtid * __kmp_stkoffset * 2;
 
             KA_TRACE( 10, ( "__kmp_create_worker: T#%d, default stacksize = %lu bytes, "
                             "__kmp_stksize = %lu bytes, final stacksize = %lu bytes\n",
@@ -1588,10 +1604,12 @@ __kmp_atfork_child (void)
     __kmp_init_common = FALSE;
 
     TCW_4(__kmp_init_user_locks, FALSE);
+#if ! KMP_USE_DYNAMIC_LOCK
     __kmp_user_lock_table.used = 1;
     __kmp_user_lock_table.allocated = 0;
     __kmp_user_lock_table.table = NULL;
     __kmp_lock_blocks = NULL;
+#endif
 
     __kmp_all_nth = 0;
     TCW_4(__kmp_nth, 0);
@@ -1712,8 +1730,8 @@ static inline void __kmp_suspend_template( int th_gtid, C *flag )
     */
     old_spin = flag->set_sleeping();
 
-    KF_TRACE( 5, ( "__kmp_suspend_template: T#%d set sleep bit for spin(%p)==%d\n",
-                   th_gtid, flag->get(), *(flag->get()) ) );
+    KF_TRACE( 5, ( "__kmp_suspend_template: T#%d set sleep bit for spin(%p)==%x, was %x\n",
+                   th_gtid, flag->get(), *(flag->get()), old_spin ) );
 
     if ( flag->done_check_val(old_spin) ) {
         old_spin = flag->unset_sleeping();
@@ -1765,8 +1783,7 @@ static inline void __kmp_suspend_template( int th_gtid, C *flag )
             status = pthread_cond_timedwait( &th->th.th_suspend_cv.c_cond, &th->th.th_suspend_mx.m_mutex, & now );
 #else
             KF_TRACE( 15, ( "__kmp_suspend_template: T#%d about to perform pthread_cond_wait\n",
-                               th_gtid ) );
-
+                            th_gtid ) );
             status = pthread_cond_wait( &th->th.th_suspend_cv.c_cond, &th->th.th_suspend_mx.m_mutex );
 #endif
 
@@ -1822,7 +1839,6 @@ void __kmp_suspend_oncore(int th_gtid, kmp_flag_oncore *flag) {
     __kmp_suspend_template(th_gtid, flag);
 }
 
-
 /* This routine signals the thread specified by target_gtid to wake up
  * after setting the sleep bit indicated by the flag argument to FALSE.
  * The target thread must already have called __kmp_suspend_template()
@@ -1856,7 +1872,7 @@ static inline void __kmp_resume_template( int target_gtid, C *flag )
         KMP_CHECK_SYSFAIL( "pthread_mutex_unlock", status );
         return;
     }
-    else {
+    else { // if multiple threads are sleeping, flag should be internally referring to a specific thread here
         typename C::flag_t old_spin = flag->unset_sleeping();
         if ( ! flag->is_sleeping_val(old_spin) ) {
             KF_TRACE( 5, ( "__kmp_resume_template: T#%d exiting, thread T#%d already awake: flag(%p): "
@@ -2253,14 +2269,14 @@ __kmp_elapsed( double *t )
 
     status = clock_gettime( CLOCK_PROCESS_CPUTIME_ID, &ts );
     KMP_CHECK_SYSFAIL_ERRNO( "clock_gettime", status );
-    *t = (double) ts.tv_nsec * (1.0 / (double) NSEC_PER_SEC) +
+    *t = (double) ts.tv_nsec * (1.0 / (double) KMP_NSEC_PER_SEC) +
         (double) ts.tv_sec;
 # else
     struct timeval tv;
 
     status = gettimeofday( & tv, NULL );
     KMP_CHECK_SYSFAIL_ERRNO( "gettimeofday", status );
-    *t = (double) tv.tv_usec * (1.0 / (double) USEC_PER_SEC) +
+    *t = (double) tv.tv_usec * (1.0 / (double) KMP_USEC_PER_SEC) +
         (double) tv.tv_sec;
 # endif
 }
@@ -2305,7 +2321,7 @@ __kmp_is_address_mapped( void * addr ) {
             if ( rc == EOF ) {
                 break;
             }; // if
-            KMP_ASSERT( rc == 3 && strlen( perms ) == 4 ); // Make sure all fields are read.
+            KMP_ASSERT( rc == 3 && KMP_STRLEN( perms ) == 4 ); // Make sure all fields are read.
 
             // Ending address is not included in the region, but beginning is.
             if ( ( addr >= beginning ) && ( addr < ending ) ) {
@@ -2480,7 +2496,7 @@ __kmp_get_load_balance( int max )
 
             // Construct task_path.
             task_path.used = task_path_fixed_len;    // Reset task_path to "/proc/".
-            __kmp_str_buf_cat( & task_path, proc_entry->d_name, strlen( proc_entry->d_name ) );
+            __kmp_str_buf_cat( & task_path, proc_entry->d_name, KMP_STRLEN( proc_entry->d_name ) );
             __kmp_str_buf_cat( & task_path, "/task", 5 );
 
             task_dir = opendir( task_path.str );
@@ -2515,7 +2531,7 @@ __kmp_get_load_balance( int max )
                         //  __kmp_str_buf_print( & stat_path, "%s/%s/stat", task_path.str, task_entry->d_name );
                         // but seriae of __kmp_str_buf_cat works a bit faster.
                         stat_path.used = stat_path_fixed_len;    // Reset stat path to its fixed part.
-                        __kmp_str_buf_cat( & stat_path, task_entry->d_name, strlen( task_entry->d_name ) );
+                        __kmp_str_buf_cat( & stat_path, task_entry->d_name, KMP_STRLEN( task_entry->d_name ) );
                         __kmp_str_buf_cat( & stat_path, "/stat", 5 );
 
                         // Note: Low-level API (open/read/close) is used. High-level API
@@ -2619,7 +2635,7 @@ __kmp_get_load_balance( int max )
 #endif // USE_LOAD_BALANCE
 
 
-#if KMP_COMPILER_GCC && !(KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_PPC64)
+#if KMP_COMPILER_GCC && !(KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_PPC64 || KMP_ARCH_AARCH64)
 
 int __kmp_invoke_microtask( microtask_t pkfn, int gtid, int tid, int argc,
         void *p_argv[] )
@@ -2655,7 +2671,7 @@ int __kmp_invoke_microtask( microtask_t pkfn, int gtid, int tid, int argc,
 
 #endif // KMP_COMPILER_GCC && !(KMP_ARCH_X86 || KMP_ARCH_X86_64 || KMP_ARCH_PPC64)
 
-#if KMP_ARCH_PPC64
+#if KMP_ARCH_PPC64 || KMP_ARCH_AARCH64
 
 // we really only need the case with 1 argument, because CLANG always build
 // a struct of pointers to shared variables referenced in the outlined function
