@@ -1,11 +1,9 @@
 /*
  * z_Windows_NT_util.c -- platform specific routines.
- * $Revision: 43389 $
- * $Date: 2014-08-11 10:54:01 -0500 (Mon, 11 Aug 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2015 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -153,9 +151,9 @@ HMODULE ntdll = NULL;
 
 /* End of NtQuerySystemInformation()-related code */
 
-#if KMP_ARCH_X86_64
+#if KMP_GROUP_AFFINITY
 static HMODULE kernel32 = NULL;
-#endif /* KMP_ARCH_X86_64 */
+#endif /* KMP_GROUP_AFFINITY */
 
 /* ----------------------------------------------------------------------------------- */
 /* ----------------------------------------------------------------------------------- */
@@ -457,7 +455,6 @@ void __kmp_suspend_oncore(int th_gtid, kmp_flag_oncore *flag) {
     __kmp_suspend_template(th_gtid, flag);
 }
 
-
 /* This routine signals the thread specified by target_gtid to wake up
  * after setting the sleep bit indicated by the flag argument to FALSE
  */
@@ -497,7 +494,6 @@ static inline void __kmp_resume_template( int target_gtid, C *flag )
         }
     }
     TCW_PTR(th->th.th_sleep_loc, NULL);
-
     KF_TRACE( 5, ( "__kmp_resume_template: T#%d about to wakeup T#%d, reset sleep bit for flag's loc(%p)\n",
                    gtid, target_gtid, flag->get() ) );
 
@@ -519,7 +515,6 @@ void __kmp_resume_oncore(int target_gtid, kmp_flag_oncore *flag) {
     __kmp_resume_template(target_gtid, flag);
 }
 
-
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
@@ -566,7 +561,7 @@ __kmp_gtid_get_specific()
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
-#if KMP_ARCH_X86_64
+#if KMP_GROUP_AFFINITY
 
 //
 // Only 1 DWORD in the mask should have any procs set.
@@ -589,15 +584,15 @@ __kmp_get_proc_group( kmp_affin_mask_t const *mask )
     return group;
 }
 
-#endif /* KMP_ARCH_X86_64 */
+#endif /* KMP_GROUP_AFFINITY */
 
 int
 __kmp_set_system_affinity( kmp_affin_mask_t const *mask, int abort_on_error )
 {
 
-#if KMP_ARCH_X86_64
+#if KMP_GROUP_AFFINITY
 
-    if (__kmp_num_proc_groups > 1) {
+        if (__kmp_num_proc_groups > 1) {
         //
         // Check for a valid mask.
         //
@@ -634,7 +629,7 @@ __kmp_set_system_affinity( kmp_affin_mask_t const *mask, int abort_on_error )
     }
     else
 
-#endif /* KMP_ARCH_X86_64 */
+#endif /* KMP_GROUP_AFFINITY */
 
     {
         if (!SetThreadAffinityMask( GetCurrentThread(), *mask )) {
@@ -657,7 +652,7 @@ int
 __kmp_get_system_affinity( kmp_affin_mask_t *mask, int abort_on_error )
 {
 
-#if KMP_ARCH_X86_64
+#if KMP_GROUP_AFFINITY
 
     if (__kmp_num_proc_groups > 1) {
         KMP_CPU_ZERO(mask);
@@ -686,7 +681,7 @@ __kmp_get_system_affinity( kmp_affin_mask_t *mask, int abort_on_error )
     }
     else
 
-#endif /* KMP_ARCH_X86_64 */
+#endif /* KMP_GROUP_AFFINITY */
 
     {
         kmp_affin_mask_t newMask, sysMask, retval;
@@ -737,7 +732,7 @@ void
 __kmp_affinity_bind_thread( int proc )
 {
 
-#if KMP_ARCH_X86_64
+#if KMP_GROUP_AFFINITY
 
     if (__kmp_num_proc_groups > 1) {
         //
@@ -766,7 +761,7 @@ __kmp_affinity_bind_thread( int proc )
     }
     else
 
-#endif /* KMP_ARCH_X86_64 */
+#endif /* KMP_GROUP_AFFINITY */
 
     {
         kmp_affin_mask_t mask;
@@ -783,10 +778,10 @@ __kmp_affinity_determine_capable( const char *env_var )
     // All versions of Windows* OS (since Win '95) support SetThreadAffinityMask().
     //
 
-#if KMP_ARCH_X86_64
-    __kmp_affin_mask_size = __kmp_num_proc_groups * sizeof(kmp_affin_mask_t);
+#if KMP_GROUP_AFFINITY
+    KMP_AFFINITY_ENABLE(__kmp_num_proc_groups*sizeof(kmp_affin_mask_t));
 #else
-    __kmp_affin_mask_size = sizeof(kmp_affin_mask_t);
+    KMP_AFFINITY_ENABLE(sizeof(kmp_affin_mask_t));
 #endif
 
     KA_TRACE( 10, (
@@ -820,7 +815,7 @@ __kmp_read_cpu_time( void )
         sec += KernelTime.dwLowDateTime;
         sec += UserTime.dwLowDateTime;
 
-        cpu_time += (sec * 100.0) / NSEC_PER_SEC;
+        cpu_time += (sec * 100.0) / KMP_NSEC_PER_SEC;
     }
 
     return cpu_time;
@@ -856,6 +851,21 @@ __kmp_runtime_initialize( void )
         return;
     };
 
+#if KMP_DYNAMIC_LIB
+    /* Pin dynamic library for the lifetime of application */
+    {
+        // First, turn off error message boxes
+        UINT err_mode = SetErrorMode (SEM_FAILCRITICALERRORS);
+        HMODULE h;
+        BOOL ret = GetModuleHandleEx( GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS
+                                     |GET_MODULE_HANDLE_EX_FLAG_PIN,
+                                     (LPCTSTR)&__kmp_serial_initialize, &h);
+        KMP_DEBUG_ASSERT2(h && ret, "OpenMP RTL cannot find itself loaded");
+        SetErrorMode (err_mode);   // Restore error mode
+        KA_TRACE( 10, ("__kmp_runtime_initialize: dynamic library pinned\n") );
+    }
+#endif
+
     InitializeCriticalSection( & __kmp_win32_section );
 #if USE_ITT_BUILD
     __kmp_itt_system_object_created( & __kmp_win32_section, "Critical Section" );
@@ -869,7 +879,7 @@ __kmp_runtime_initialize( void )
     #endif /* KMP_ARCH_X86 || KMP_ARCH_X86_64 */
 
     /* Set up minimum number of threads to switch to TLS gtid */
-    #if KMP_OS_WINDOWS && ! defined GUIDEDLL_EXPORTS
+    #if KMP_OS_WINDOWS && ! defined KMP_DYNAMIC_LIB
         // Windows* OS, static library.
         /*
             New thread may use stack space previously used by another thread, currently terminated.
@@ -880,7 +890,7 @@ __kmp_runtime_initialize( void )
             foreign tread.
 
             Setting __kmp_tls_gtid_min to 0 workarounds this problem: __kmp_get_global_thread_id()
-            does not search through stacks, but get gtid from TLS immediatelly.
+            does not search through stacks, but get gtid from TLS immediately.
 
             --ln
         */
@@ -939,7 +949,7 @@ __kmp_runtime_initialize( void )
     }
     KMP_DEBUG_ASSERT( NtQuerySystemInformation != NULL );
 
-#if KMP_ARCH_X86_64
+#if KMP_GROUP_AFFINITY
     //
     // Load kernel32.dll.
     // Same caveat - must use full system path name.
@@ -956,6 +966,7 @@ __kmp_runtime_initialize( void )
         // Load kernel32.dll using full path.
         //
         kernel32 = GetModuleHandle( path.str );
+        KA_TRACE( 10, ("__kmp_runtime_initialize: kernel32.dll = %s\n", path.str ) );
 
         //
         // Load the function pointers to kernel32.dll routines
@@ -967,6 +978,12 @@ __kmp_runtime_initialize( void )
             __kmp_GetThreadGroupAffinity = (kmp_GetThreadGroupAffinity_t) GetProcAddress( kernel32, "GetThreadGroupAffinity" );
             __kmp_SetThreadGroupAffinity = (kmp_SetThreadGroupAffinity_t) GetProcAddress( kernel32, "SetThreadGroupAffinity" );
 
+            KA_TRACE( 10, ("__kmp_runtime_initialize: __kmp_GetActiveProcessorCount = %p\n", __kmp_GetActiveProcessorCount ) );
+            KA_TRACE( 10, ("__kmp_runtime_initialize: __kmp_GetActiveProcessorGroupCount = %p\n", __kmp_GetActiveProcessorGroupCount ) );
+            KA_TRACE( 10, ("__kmp_runtime_initialize:__kmp_GetThreadGroupAffinity = %p\n", __kmp_GetThreadGroupAffinity ) );
+            KA_TRACE( 10, ("__kmp_runtime_initialize: __kmp_SetThreadGroupAffinity = %p\n", __kmp_SetThreadGroupAffinity ) );
+            KA_TRACE( 10, ("__kmp_runtime_initialize: sizeof(kmp_affin_mask_t) = %d\n", sizeof(kmp_affin_mask_t) ) );
+
             //
             // See if group affinity is supported on this system.
             // If so, calculate the #groups and #procs.
@@ -992,9 +1009,12 @@ __kmp_runtime_initialize( void )
                 for ( i = 0; i < __kmp_num_proc_groups; i++ ) {
                     DWORD size = __kmp_GetActiveProcessorCount( i );
                     __kmp_xproc += size;
-                    KA_TRACE( 20, ("__kmp_runtime_initialize: proc group %d size = %d\n", i, size ) );
+                    KA_TRACE( 10, ("__kmp_runtime_initialize: proc group %d size = %d\n", i, size ) );
                 }
             }
+            else {
+                KA_TRACE( 10, ("__kmp_runtime_initialize: %d processor groups detected\n", __kmp_num_proc_groups ) );
+            }
         }
     }
     if ( __kmp_num_proc_groups <= 1 ) {
@@ -1004,7 +1024,7 @@ __kmp_runtime_initialize( void )
 #else
     GetSystemInfo( & info );
     __kmp_xproc = info.dwNumberOfProcessors;
-#endif // KMP_ARCH_X86_64
+#endif /* KMP_GROUP_AFFINITY */
 
     //
     // If the OS said there were 0 procs, take a guess and use a value of 2.
@@ -1148,46 +1168,6 @@ __kmp_read_system_time( double *delta )
     }
 }
 
-/* ------------------------------------------------------------------------ */
-/* ------------------------------------------------------------------------ */
-
-/*
- * Change thread to the affinity mask pointed to by affin_mask argument
- * and return a pointer to the old value in the old_mask argument, if argument
- * is non-NULL.
- */
-
-void
-__kmp_change_thread_affinity_mask( int gtid, kmp_affin_mask_t *new_mask,
-                                   kmp_affin_mask_t *old_mask )
-{
-    kmp_info_t  *th = __kmp_threads[ gtid ];
-
-    KMP_DEBUG_ASSERT( *new_mask != 0 );
-
-    if ( old_mask != NULL ) {
-        *old_mask = SetThreadAffinityMask( th -> th.th_info.ds.ds_thread, *new_mask );
-
-        if (! *old_mask ) {
-            DWORD error = GetLastError();
-            __kmp_msg(
-                kmp_ms_fatal,
-                KMP_MSG( CantSetThreadAffMask ),
-                KMP_ERR( error ),
-                __kmp_msg_null
-            );
-        }
-    }
-    if (__kmp_affinity_verbose)
-            KMP_INFORM( ChangeAffMask, "KMP_AFFINITY (Bind)", gtid, *old_mask, *new_mask );
-
-    /* Make sure old value is correct in thread data structures */
-    KMP_DEBUG_ASSERT( old_mask != NULL && *old_mask == *(th -> th.th_affin_mask ));
-
-    KMP_CPU_COPY(th -> th.th_affin_mask, new_mask);
-}
-
-
 /* ------------------------------------------------------------------------ */
 /* ------------------------------------------------------------------------ */
 
@@ -1226,7 +1206,7 @@ __kmp_launch_worker( void *arg )
 #endif /* KMP_ARCH_X86 || KMP_ARCH_X86_64 */
 
     if ( __kmp_stkoffset > 0 && gtid > 0 ) {
-        padding = _alloca( gtid * __kmp_stkoffset );
+        padding = KMP_ALLOCA( gtid * __kmp_stkoffset );
     }
 
     KMP_FSYNC_RELEASING( &this_thr -> th.th_info.ds.ds_alive );
