@@ -1,7 +1,7 @@
 #!/usr/bin/env perl
 
 # <copyright>
-#    Copyright (c) 2013-2014 Intel Corporation.  All Rights Reserved.
+#    Copyright (c) 2013-2015 Intel Corporation.  All Rights Reserved.
 #
 #    Redistribution and use in source and binary forms, with or without
 #    modification, are permitted provided that the following conditions
@@ -114,10 +114,16 @@ sub get_deps_ldd($) {
 sub get_deps_readelf($) {
 
     my $file = shift ( @_ );
-    my $tool = "readelf";
+    my $tool;
     my @bulk;
     my @deps;
 
+    if($target_arch eq "mic") {
+        $tool = "x86_64-k1om-linux-readelf";
+    } else {
+        $tool = "readelf";
+    }
+
     execute( [ $tool, "-d", $file ], -stdout => \@bulk );
     debug( @bulk, "(eof)" );
 
@@ -365,7 +371,7 @@ if ( not -e $lib ){
 
 # Select appropriate get_deps implementation.
 if ( 0 ) {
-} elsif ( $target_os eq "lin" or $target_os eq "lrb" ) {
+} elsif ( $target_os eq "lin" ) {
     *get_deps = \*get_deps_readelf;
 } elsif ( $target_os eq "mac" ) {
     *get_deps = \*get_deps_otool;
