@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 
 # <copyright>
-#    Copyright (c) 2013-2014 Intel Corporation.  All Rights Reserved.
+#    Copyright (c) 2013-2015 Intel Corporation.  All Rights Reserved.
 #
 #    Redistribution and use in source and binary forms, with or without
 #    modification, are permitted provided that the following conditions
@@ -37,6 +37,7 @@ use FindBin;
 use lib "$FindBin::Bin/lib";
 
 use tools;
+use Platform ":vars";
 
 our $VERSION = "0.002";
 
@@ -44,7 +45,13 @@ sub execstack($) {
     my ( $file ) = @_;
     my @output;
     my @stack;
-    execute( [ "readelf", "-l", "-W", $file ], -stdout => \@output );
+    my $tool;
+    if($target_arch eq "mic") {
+        $tool = "x86_64-k1om-linux-readelf";
+    } else {
+        $tool = "readelf";
+    }
+    execute( [ $tool, "-l", "-W", $file ], -stdout => \@output );
     @stack = grep( $_ =~ m{\A\s*(?:GNU_)?STACK\s+}, @output );
     if ( not @stack ) {
         # Interpret missed "STACK" line as error.
@@ -68,6 +75,7 @@ sub execstack($) {
 }; # sub execstack
 
 get_options(
+    Platform::target_options(),
 );
 
 foreach my $file ( @ARGV ) {
