@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 
 # <copyright>
-#    Copyright (c) 2013-2014 Intel Corporation.  All Rights Reserved.
+#    Copyright (c) 2013-2015 Intel Corporation.  All Rights Reserved.
 #
 #    Redistribution and use in source and binary forms, with or without
 #    modification, are permitted provided that the following conditions
@@ -209,7 +209,7 @@ sub get_intel_compiler_version($) {
     if ( not defined( $ic_arch ) ) {
         return @ret;
     }; # if
-    if ( Platform::canon_arch( $ic_arch ) ne $target_arch ) {
+    if ( Platform::canon_arch( $ic_arch ) ne $target_arch and not (Platform::canon_arch($ic_arch) eq "32e" and $target_arch eq "mic" )) {
         warning( "Target architecture is $target_arch, $tool for $ic_arch found." );
         return @ret;
     }; # if
@@ -410,7 +410,6 @@ my $fortran     = 0;             # Check for corresponding Fortran compiler, ifo
 my $clang       = 0;             # Check Clang Compilers.
 my $intel_compilers = {
     "lin" => { c => "icc", cpp => "icpc", f => "ifort" },
-    "lrb" => { c => "icc", cpp => "icpc", f => "ifort" },
     "mac" => { c => "icc", cpp => "icpc", f => "ifort" },
     "win" => { c => "icl", cpp => undef,  f => "ifort" },
 };
