@@ -1,7 +1,7 @@
 #!/usr/bin/env perl
 
 # <copyright>
-#    Copyright (c) 2013-2014 Intel Corporation.  All Rights Reserved.
+#    Copyright (c) 2013-2015 Intel Corporation.  All Rights Reserved.
 #
 #    Redistribution and use in source and binary forms, with or without
 #    modification, are permitted provided that the following conditions
@@ -160,7 +160,7 @@ if ( not -w $output ) {
 
 if ( $target_os eq "win" ) {
     *process = \&windows;
-} elsif ( $target_os eq "lin" or $target_os eq "lrb" ) {
+} elsif ( $target_os eq "lin") {
     *process = \&linux;
 } elsif ( $target_os eq "mac" ) {
     *process = \&darwin;
